# PYTHON SOFTWARE FOUNDATION LICENSE VERSION 2
# --------------------------------------------
#
# 1. This LICENSE AGREEMENT is between the Python Software Foundation
# ("PSF"), and the Individual or Organization ("Licensee") accessing and
# otherwise using this software ("Python") in source or binary form and
# its associated documentation.
#
# 2. Subject to the terms and conditions of this License Agreement, PSF hereby
# grants Licensee a nonexclusive, royalty-free, world-wide license to reproduce,
# analyze, test, perform and/or display publicly, prepare derivative works,
# distribute, and otherwise use Python alone or in any derivative version,
# provided, however, that PSF's License Agreement and PSF's notice of copyright,
# i.e., "Copyright (c) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010,
# 2011, 2012, 2013, 2014 Python Software Foundation; All Rights Reserved" are
# retained in Python alone or in any derivative version prepared by Licensee.
#
# 3. In the event Licensee prepares a derivative work that is based on
# or incorporates Python or any part thereof, and wants to make
# the derivative work available to others as provided herein, then
# Licensee hereby agrees to include in any such work a brief summary of
# the changes made to Python.
#
# 4. PSF is making Python available to Licensee on an "AS IS"
# basis.  PSF MAKES NO REPRESENTATIONS OR WARRANTIES, EXPRESS OR
# IMPLIED.  BY WAY OF EXAMPLE, BUT NOT LIMITATION, PSF MAKES NO AND
# DISCLAIMS ANY REPRESENTATION OR WARRANTY OF MERCHANTABILITY OR FITNESS
# FOR ANY PARTICULAR PURPOSE OR THAT THE USE OF PYTHON WILL NOT
# INFRINGE ANY THIRD PARTY RIGHTS.
#
# 5. PSF SHALL NOT BE LIABLE TO LICENSEE OR ANY OTHER USERS OF PYTHON
# FOR ANY INCIDENTAL, SPECIAL, OR CONSEQUENTIAL DAMAGES OR LOSS AS
# A RESULT OF MODIFYING, DISTRIBUTING, OR OTHERWISE USING PYTHON,
# OR ANY DERIVATIVE THEREOF, EVEN IF ADVISED OF THE POSSIBILITY THEREOF.
#
# 6. This License Agreement will automatically terminate upon a material
# breach of its terms and conditions.
#
# 7. Nothing in this License Agreement shall be deemed to create any
# relationship of agency, partnership, or joint venture between PSF and
# Licensee.  This License Agreement does not grant permission to use PSF
# trademarks or trade name in a trademark sense to endorse or promote
# products or services of Licensee, or any third party.
#
# 8. By copying, installing or otherwise using Python, Licensee
# agrees to be bound by the terms and conditions of this License
# Agreement.
#
# Original Python Recipe for Proxy:
# http://code.activestate.com/recipes/496741-object-proxying/
# Author: Tomer Filiba

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

import json

from ansible.utils.unicode import to_unicode
from ansible.compat.six import string_types, text_type

__all__ = ['UnsafeProxy', 'AnsibleUnsafe', 'AnsibleJSONUnsafeEncoder', 'AnsibleJSONUnsafeDecoder', 'wrap_var']

class AnsibleUnsafe(object):
    __UNSAFE__ = True

class AnsibleUnsafeText(text_type, AnsibleUnsafe):
    # def __getattribute__(self, name):
    #     print('attr: %s' % name)
    #     return object.__getattribute__(self, name)

    def _strip_unsafe(self):
        return super(AnsibleUnsafeText, self).__str__()

    def __str__(self):  # pylint: disable=invalid-str-returned
        return self

    def __repr__(self):  # pylint: disable=invalid-repr-returned
        return self.__class__(super(AnsibleUnsafeText, self).__repr__())

    def __format__(self, format_spec):  # pylint: disable=invalid-format-returned
        return self.__class__(super(AnsibleUnsafeText, self).__format__(format_spec))

    def __getitem__(self, key):
        return self.__class__(super(AnsibleUnsafeText, self).__getitem__(key))

    def __iter__(self):
        class Iterator(object):
            def __init__(self, string, unsafe_text_cls):
                self.string = string
                self.unsafe_text_cls = unsafe_text_cls
                self.index = 0
                self.length = len(string)

            def __iter__(self):
                return self

            def next(self):
                if self.index >= self.length:
                    raise StopIteration
                result = self.unsafe_text_cls(self.string[self.index])
                self.index += 1
                return result

        return Iterator(self, AnsibleUnsafeText)

    def __reversed__(self):
        return self[::-1]

    def __add__(self, value):
        return self.__class__(super(AnsibleUnsafeText, self).__add__(value))

    def __radd__(self, value):
        return self.__class__(value.__add__(self))

    def __mul__(self, value):
        return self.__class__(super(AnsibleUnsafeText, self).__mul__(value))

    __rmul__ = __mul__

    def __mod__(self, value):
        return self.__class__(super(AnsibleUnsafeText, self).__mod__(value))

    def __rmod__(self, value):
        return self.__class__(super(AnsibleUnsafeText, self).__rmod__(value))

    def capitalize(self):
        return self.__class__(super(AnsibleUnsafeText, self).capitalize())

    def casefold(self):
        # casefold does not exist for unicode characters in python2.7.17, lower gives approximately
        # the same results. https://www.w3.org/TR/charmod-norm/#definitionCaseFolding
        return self.__class__(super(AnsibleUnsafeText, self).lower())

    def center(self, width, fillchar=' '):
        return self.__class__(super(AnsibleUnsafeText, self).center(width, fillchar))

    def removeprefix(self, prefix):
        if self.startswith(prefix):
            return self.__class__(self[len(prefix):])
        return self.__class__(self)

    def removesuffix(self, suffix):
        if self.endswith(suffix):
            return self.__class__(self[:-len(suffix)])
        return self.__class__(self)

    def expandtabs(self, tabsize=8):
        return self.__class__(super(AnsibleUnsafeText, self).expandtabs(tabsize))

    def format(self, *args, **kwargs):
        return self.__class__(super(AnsibleUnsafeText, self).format(*args, **kwargs))

    def format_map(self, mapping):
        return self.__class__(super(AnsibleUnsafeText, self).format(**mapping))

    def join(self, iterable):
        return self.__class__(super(AnsibleUnsafeText, self).join(iterable))

    def ljust(self, width, fillchar=' '):
        return self.__class__(super(AnsibleUnsafeText, self).ljust(width, fillchar))

    def lower(self):
        return self.__class__(super(AnsibleUnsafeText, self).lower())

    def lstrip(self, chars=None):
        return self.__class__(super(AnsibleUnsafeText, self).lstrip(chars))

    def partition(self, sep):
        cls = self.__class__
        return tuple(cls(e) for e in super(AnsibleUnsafeText, self).partition(sep))

    def replace(self, old, new, count=-1):
        return self.__class__(super(AnsibleUnsafeText, self).replace(old, new, count))

    def rjust(self, width, fillchar=' '):
        return self.__class__(super(AnsibleUnsafeText, self).rjust(width, fillchar))

    def rpartition(self, sep):
        cls = self.__class__
        return tuple(cls(e) for e in super(AnsibleUnsafeText, self).rpartition(sep))

    def rstrip(self, chars=None):
        return self.__class__(super(AnsibleUnsafeText, self).rstrip(chars))

    def split(self, sep=None, maxsplit=-1):
        cls = self.__class__
        return [cls(e) for e in super(AnsibleUnsafeText, self).split(sep, maxsplit)]

    def rsplit(self, sep=None, maxsplit=-1):
        cls = self.__class__
        return [cls(e) for e in super(AnsibleUnsafeText, self).rsplit(sep, maxsplit)]

    def splitlines(self, keepends=False):
        cls = self.__class__
        return [cls(e) for e in super(AnsibleUnsafeText, self).splitlines(keepends)]

    def strip(self, chars=None):
        return self.__class__(super(AnsibleUnsafeText, self).strip(chars))

    def swapcase(self):
        return self.__class__(super(AnsibleUnsafeText, self).swapcase())

    def title(self):
        return self.__class__(super(AnsibleUnsafeText, self).title())

    def translate(self, table):
        return self.__class__(super(AnsibleUnsafeText, self).translate(table))

    def upper(self):
        return self.__class__(super(AnsibleUnsafeText, self).upper())

    def zfill(self, width):
        return self.__class__(super(AnsibleUnsafeText, self).zfill(width))

class UnsafeProxy(object):
    def __new__(cls, obj, *args, **kwargs):
        # In our usage we should only receive unicode strings.
        # This conditional and conversion exists to sanity check the values
        # we're given but we may want to take it out for testing and sanitize
        # our input instead.
        if isinstance(obj, string_types):
            obj = to_unicode(obj, errors='strict')
            return AnsibleUnsafeText(obj)
        return obj

class AnsibleJSONUnsafeEncoder(json.JSONEncoder):
    def encode(self, obj):
        if isinstance(obj, AnsibleUnsafe):
            return super(AnsibleJSONUnsafeEncoder, self).encode(dict(__ansible_unsafe=True, value=unicode(obj)))
        else:
            return super(AnsibleJSONUnsafeEncoder, self).encode(obj)

class AnsibleJSONUnsafeDecoder(json.JSONDecoder):
    def decode(self, obj):
        value = super(AnsibleJSONUnsafeDecoder, self).decode(obj)
        if isinstance(value, dict) and '__ansible_unsafe' in value:
            return UnsafeProxy(value.get('value', ''))
        else:
            return value

def _wrap_dict(v):
    for k in v.keys():
        if v[k] is not None:
            v[wrap_var(k)] = wrap_var(v[k])
    return v


def _wrap_list(v):
    for idx, item in enumerate(v):
        if item is not None:
            v[idx] = wrap_var(item)
    return v


def wrap_var(v):
    if isinstance(v, dict):
        v = _wrap_dict(v)
    elif isinstance(v, list):
        v = _wrap_list(v)
    else:
        if v is not None and not isinstance(v, AnsibleUnsafe):
            v = UnsafeProxy(v)
    return v

def _is_unsafe(obj):
    return getattr(obj, '__UNSAFE__', False) is True
